/*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/********************************************************************
* Author: Virtualik <virty1k@mail.ru>
* WWW   : http://virtualsoft.narod.ru
* Copyright (c) 2003,2006 Virtualik  All rights reserved.
********************************************************************/

#include "sndplay.h"
#include <stdio.h>

#ifndef EXTPLAYER

int dspdevice, done = 0;

void
stopplay (int x)
{
  done = 1;
}

int
PrepareDSP (char *dspname)
{
  if ((dspdevice = open (dspname, O_WRONLY)) == -1)
    return EPSDSPNOTREADY;
  return 0;
}

void
CloseDSP ()
{
  if (dspdevice != 0)
    {
      ioctl (dspdevice, SNDCTL_DSP_RESET, 0);
      close (dspdevice);
    }
}

#endif

int
PlaySound (char *szFilename)
{
#ifndef EXTPLAYER
  char *bufdata;
  int len, rsize, buf_size, datalen = 0;
  riff_header riffinfo;
  waveheader wformat;

  if (dspdevice == 0)
    return EPSDSPNOTREADY;

  int fd, res = 0;
  if ((fd = open (szFilename, O_RDONLY)) == -1)
    return EPSOPEN;

  read (fd, &riffinfo, sizeof (riff_header));
  if (riffinfo.chunk_type != FOURCC_WAVE)
    {
      res = EPSNOTWAVE;
      goto doneplay;
    }

  read (fd, &wformat, riffinfo.chunk_length);
  if (wformat.format != PCM_WAVE_FORMAT)
    {
      res = EPSNOTPCM;
      goto doneplay;
    }

  datalen = 0;
  rsize = 0;
  do {
   if (datalen > 0) // Skip non-worthy section
    lseek (fd, datalen, SEEK_CUR);
   read(fd, &rsize, sizeof (int));
   read (fd, &datalen, sizeof (int));
  } while(rsize != FOURCC_DATA);
  
  ioctl (dspdevice, SNDCTL_DSP_RESET, 0);

  if (ioctl (dspdevice, SNDCTL_DSP_GETBLKSIZE, &buf_size) < 0)
    {
      res = EPSDSPSETUP1;
      goto doneplay;
    }

  rsize = wformat.precision;
  if (ioctl (dspdevice, SNDCTL_DSP_SETFMT, &rsize) < 0)
    {
      res = EPSDSPSETUP2;
      goto doneplay;
    }

  if (ioctl (dspdevice, SNDCTL_DSP_SPEED, &wformat.sample_rate) < 0)
    {
      res = EPSDSPSETUP3;
      goto doneplay;
    }

  if (wformat.channels > 1)
    {
      if (ioctl (dspdevice, SNDCTL_DSP_STEREO, &wformat.channels) < 0)
	{
	  res = EPSDSPSETUP4;
	  goto doneplay;
	}
    }

  // As it seems this does nothing ;(
  if (ioctl (dspdevice, SNDCTL_DSP_SYNC, NULL) < 0)
    {
      res = EPSDSPSETUP5;
      goto doneplay;
    }

  /* Reading data and throwing it to DSP */
  bufdata = (char *) malloc (buf_size);

  signal (SIGINT, stopplay);

  len = datalen;
  while (len > 0)
    {
      if (done)
	break;
      if ((rsize = read (fd, bufdata, buf_size)) < 0)
	{
	  res = EPSOPEN;
	  goto doneplay2;
	}
      if (write (dspdevice, bufdata, rsize) < 0)
	{
	  res = EPSDSPNOTREADY;
	  goto doneplay2;
	}
      len -= rsize;
    }
#ifdef __FreeBSD__
  float div = datalen;
  div /= wformat.speed;
  div *= 1000000; // Get sound length in seconds
  div += 50000; // Add 0.05 second
  usleep (div);
#else
  usleep (50000);
#endif

doneplay2:
  free (bufdata);

doneplay:
  close (fd);
  return res;
#else
 char snd[PATH_MAX] = {0};
 strcpy(snd, EXTPLAYER);
 strncat(snd, " ", 1);
 strcat(snd, szFilename);
 return system(snd);
#endif
}
